// Code generated by informer-gen. DO NOT EDIT.

package v1

import (
	internalinterfaces "github.com/dask/dask-kubernetes/v2026/dask_kubernetes/operator/go_client/pkg/client/informers/externalversions/internalinterfaces"
)

// Interface provides access to all the informers in this group version.
type Interface interface {
	// DaskAutoscalers returns a DaskAutoscalerInformer.
	DaskAutoscalers() DaskAutoscalerInformer
	// DaskClusters returns a DaskClusterInformer.
	DaskClusters() DaskClusterInformer
	// DaskJobs returns a DaskJobInformer.
	DaskJobs() DaskJobInformer
	// DaskWorkerGroups returns a DaskWorkerGroupInformer.
	DaskWorkerGroups() DaskWorkerGroupInformer
}

type version struct {
	factory          internalinterfaces.SharedInformerFactory
	namespace        string
	tweakListOptions internalinterfaces.TweakListOptionsFunc
}

// New returns a new Interface.
func New(f internalinterfaces.SharedInformerFactory, namespace string, tweakListOptions internalinterfaces.TweakListOptionsFunc) Interface {
	return &version{factory: f, namespace: namespace, tweakListOptions: tweakListOptions}
}

// DaskAutoscalers returns a DaskAutoscalerInformer.
func (v *version) DaskAutoscalers() DaskAutoscalerInformer {
	return &daskAutoscalerInformer{factory: v.factory, namespace: v.namespace, tweakListOptions: v.tweakListOptions}
}

// DaskClusters returns a DaskClusterInformer.
func (v *version) DaskClusters() DaskClusterInformer {
	return &daskClusterInformer{factory: v.factory, namespace: v.namespace, tweakListOptions: v.tweakListOptions}
}

// DaskJobs returns a DaskJobInformer.
func (v *version) DaskJobs() DaskJobInformer {
	return &daskJobInformer{factory: v.factory, namespace: v.namespace, tweakListOptions: v.tweakListOptions}
}

// DaskWorkerGroups returns a DaskWorkerGroupInformer.
func (v *version) DaskWorkerGroups() DaskWorkerGroupInformer {
	return &daskWorkerGroupInformer{factory: v.factory, namespace: v.namespace, tweakListOptions: v.tweakListOptions}
}
