// Code generated by client-gen. DO NOT EDIT.

package v1

type DaskAutoscalerExpansion interface{}

type DaskClusterExpansion interface{}

type DaskJobExpansion interface{}

type DaskWorkerGroupExpansion interface{}
