// Code generated by lister-gen. DO NOT EDIT.

package v1

// DaskAutoscalerListerExpansion allows custom methods to be added to
// DaskAutoscalerLister.
type DaskAutoscalerListerExpansion interface{}

// DaskAutoscalerNamespaceListerExpansion allows custom methods to be added to
// DaskAutoscalerNamespaceLister.
type DaskAutoscalerNamespaceListerExpansion interface{}

// DaskClusterListerExpansion allows custom methods to be added to
// DaskClusterLister.
type DaskClusterListerExpansion interface{}

// DaskClusterNamespaceListerExpansion allows custom methods to be added to
// DaskClusterNamespaceLister.
type DaskClusterNamespaceListerExpansion interface{}

// DaskJobListerExpansion allows custom methods to be added to
// DaskJobLister.
type DaskJobListerExpansion interface{}

// DaskJobNamespaceListerExpansion allows custom methods to be added to
// DaskJobNamespaceLister.
type DaskJobNamespaceListerExpansion interface{}

// DaskWorkerGroupListerExpansion allows custom methods to be added to
// DaskWorkerGroupLister.
type DaskWorkerGroupListerExpansion interface{}

// DaskWorkerGroupNamespaceListerExpansion allows custom methods to be added to
// DaskWorkerGroupNamespaceLister.
type DaskWorkerGroupNamespaceListerExpansion interface{}
