// Code generated by informer-gen. DO NOT EDIT.

package externalversions

import (
	"fmt"

	v1 "github.com/dask/dask-kubernetes/v2026/dask_kubernetes/operator/go_client/pkg/apis/kubernetes.dask.org/v1"
	schema "k8s.io/apimachinery/pkg/runtime/schema"
	cache "k8s.io/client-go/tools/cache"
)

// GenericInformer is type of SharedIndexInformer which will locate and delegate to other
// sharedInformers based on type
type GenericInformer interface {
	Informer() cache.SharedIndexInformer
	Lister() cache.GenericLister
}

type genericInformer struct {
	informer cache.SharedIndexInformer
	resource schema.GroupResource
}

// Informer returns the SharedIndexInformer.
func (f *genericInformer) Informer() cache.SharedIndexInformer {
	return f.informer
}

// Lister returns the GenericLister.
func (f *genericInformer) Lister() cache.GenericLister {
	return cache.NewGenericLister(f.Informer().GetIndexer(), f.resource)
}

// ForResource gives generic access to a shared informer of the matching type
// TODO extend this to unknown resources with a client pool
func (f *sharedInformerFactory) ForResource(resource schema.GroupVersionResource) (GenericInformer, error) {
	switch resource {
	// Group=kubernetes.dask.org, Version=v1
	case v1.SchemeGroupVersion.WithResource("daskautoscalers"):
		return &genericInformer{resource: resource.GroupResource(), informer: f.Kubernetes().V1().DaskAutoscalers().Informer()}, nil
	case v1.SchemeGroupVersion.WithResource("daskclusters"):
		return &genericInformer{resource: resource.GroupResource(), informer: f.Kubernetes().V1().DaskClusters().Informer()}, nil
	case v1.SchemeGroupVersion.WithResource("daskjobs"):
		return &genericInformer{resource: resource.GroupResource(), informer: f.Kubernetes().V1().DaskJobs().Informer()}, nil
	case v1.SchemeGroupVersion.WithResource("daskworkergroups"):
		return &genericInformer{resource: resource.GroupResource(), informer: f.Kubernetes().V1().DaskWorkerGroups().Informer()}, nil

	}

	return nil, fmt.Errorf("no informer found for %v", resource)
}
